/***********************************************************

	LCD-Routinen
	fur Samsung 2138a mit M50530-Controller
	24 x 8 Zeichen

*************************************************************/





#include "stm32f10x.h"
#include "m50530.h"
#include "math.h"
#include "DelaySysTick.h"

extern void Delay_ms(uint32_t nTime);

// sendet den angelegten Befehl zum LCD
// setzt kurzzeitig das EX-Signal und nimmt es anschlie?end wieder zuruck
void LCD_execute(void)
{
	Delay_100us(2);
	M50530_E_On;
	Delay_100us(2);
	M50530_E_Off;
}
//

// wartet solange bis das Busy-Flag nicht mehr gesetzt ist
// und das LCD weitere Befehle entgegennimmt
uint8_t LCD_waitReady(void)
{
	uint32_t stat = 0 ;
	uint32_t tempB = 0 ;
	uint32_t tempA = 0 ;
	M50530_D4_7_INPUT;
	M50530_CS1_Off;
	M50530_CS2_Off;
	M50530_RnW_On;	
	while(1)
	{
		Delay_us(100);
		M50530_E_On;
		Delay_us(100);
		tempB = GPIOB->IDR;
		tempA = GPIOA->IDR;
		M50530_E_Off;
		Delay_us(100);
		if((tempB & GPIO_IDR_IDR13)!=0){continue;}	// test if BUSY bit set
		if((tempB & GPIO_IDR_IDR12)==0)							// test if 4u2 bit set
		{
			if((tempA & GPIO_IDR_IDR12)!=0){stat=(1<<5);}else{stat=0;}	// 8/~4 bit mode bit5 of stat
			if((tempB & GPIO_IDR_IDR11)!=0){stat|=(1<<4);}	// 8/~12 font bit4 of stat
			if((tempA & GPIO_IDR_IDR12)!=0){break;}	// test if I/O 8/4 bit set
			else{continue;}													// I/O 4 bit set, ex RB 2nd half for 4 bit mode
		}
		else
		{
			stat|=(1<<2);	// stat 4u2 = 1 - second half of semibyte in 4 bit mode
			if((tempA & GPIO_IDR_IDR12)!=0){	stat|=(1<<1);}	// stat RA1 = 1
			if((tempB & GPIO_IDR_IDR11)!=0){	stat|=(1<<0);}	// stat RA0 = 1
			break;
		}
	}
	M50530_RnW_Off;
	M50530_D4_7_OUTPUT;
	return stat ;
}
//

// ubertragt das ubergebene Byte an das LCD
void LCD_sendByte(const unsigned char ddata)
{
	M50530_D4_7_OUTPUT;
	
	Delay_us(100);

	if (ddata & (1<<4)) M50530_D4_On;
	else M50530_D4_Off;
	
	if (ddata & (1<<5)) M50530_D5_On;
	else M50530_D5_Off;
	
	if (ddata & (1<<6)) M50530_D6_On;
	else M50530_D6_Off;
	
	if (ddata & (1<<7)) M50530_D7_On;
	else M50530_D7_Off;	
	
	Delay_us(100);
	LCD_execute();
	Delay_us(100);

	
	if (ddata & (1<<0)) M50530_D4_On;
	else M50530_D4_Off;
	
	if (ddata & (1<<1)) M50530_D5_On;
	else M50530_D5_Off;
	
	if (ddata & (1<<2)) M50530_D6_On;
	else M50530_D6_Off;
	
	if (ddata & (1<<3)) M50530_D7_On;
	else M50530_D7_Off;

	Delay_us(100);
	LCD_execute();
	Delay_us(100);
	M50530_D4_Off;
	M50530_D5_Off;
	M50530_D6_Off;
	M50530_D7_Off;
	
	
}
//

void LCD_sendUpperHalfByte(const unsigned char ddata)
{
	M50530_D4_7_OUTPUT;
	
	Delay_us(100);

	if (ddata & (1<<4)) M50530_D4_On;
	else M50530_D4_Off;
	
	if (ddata & (1<<5)) M50530_D5_On;
	else M50530_D5_Off;
	
	if (ddata & (1<<6)) M50530_D6_On;
	else M50530_D6_Off;
	
	if (ddata & (1<<7)) M50530_D7_On;
	else M50530_D7_Off;	
	
	Delay_us(100);
	LCD_execute();
	Delay_us(100);
	M50530_D4_Off;
	M50530_D5_Off;
	M50530_D6_Off;
	M50530_D7_Off;
}
//


// loscht die Anzeige und setzt die Display + Cursor-Adresse auf 0,0
void LCD_clear(void)
{
	/*
		CLEAR DISPLAY, MOVE DISPLAY/CURSOR ADDRESS HOME
		input:
			OC1		0
			OC2		0
			RW		0
			DB7-DB1	0
			DB0		1
	*/
	M50530_CS1_Off;
	M50530_CS2_Off;
	M50530_RnW_Off;
	LCD_sendByte(1);	// DB0
	Delay_us(100);
}


// setzt 4-Bit-Mode und initialisiert das LCD
// muss als erstes aufgerufen werden, bevor das LCD angesteuert werden kann
void LCD_init(void)
{
	uint8_t temp = 0 ;

	// auf 4-Bit-Mode umschalten
	/*
		SET FUNCTION MODE
		input:
			OC1		0
			OC2		0
			RW		0
			DB7,DB6	1
			DB5		I/O 8/4	(1 = 8 bit, 0 = 4 bit)
			DB4		FONT 8/12 (1 = 5x8 fonts, 0 = 5x12 fonts)
			DB3,DB2	DUTY
			DB1,DB0	RAM
	*/

	temp = LCD_waitReady();
	if ((temp & 1<<5)==0){GRN_LED_On;RED_LED_Off;}
	else{GRN_LED_Off;RED_LED_On;LCD_sendUpperHalfByte(0xC0);} // set 4 bit mode if needeed

	M50530_CS1_On;
	M50530_CS2_On;
	Delay_100us(1);
	M50530_CS1_Off;
	M50530_CS2_Off;
	M50530_RnW_Off;
	
	

	temp = LCD_waitReady();
	if ((temp & 1<<5)==0){GRN_LED_On;RED_LED_Off;}
	else{GRN_LED_Off;RED_LED_On;}
	
	LCD_waitReady();
	
	LCD_sendByte(0xDB);					//	SF: 4 bit 4 line 5x8 font
	Delay_ms(2);
	LCD_sendByte(0x01);					// 	CH: cursor home
	Delay_ms(2);
	LCD_sendByte(0x3F);					//	SD: cursor on
	Delay_ms(2);
	LCD_sendByte(0x58);					//	SE: WD address auto inc
	Delay_ms(2);

	
//	LCD_sendByte(216);	// DB7, DB6, DB4, DB3
//	LCD_clear();
}


// schreibt die ubergebene Zeichenkette an die aktuelle Cursor-Position
void LCD_write(const char * c)
{
    while (*c) 
      LCD_writeChar(*c++);
}


// schreibt die ubergebene Zeichenkette an die aktuelle Cursor-Position (fur String im ROM)
void LCD_write_P(const unsigned char * progmem_string)
{
//	register unsigned char c;
//	while ((c = pgm_read_byte(progmem_string++)))
//		LCD_writeChar(c);
}


// setzt den Cursor an die angegebene Position (y = Zeile, x = Spalte)
void LCD_setCursorPos(const unsigned char y, const unsigned char x)
{
	// Adresse aus x und y berechnen
	// Adressen sind wie folgt: Zeile 0 = 0; Zeile 1 = 40; Zeile 2 = 128; Zeile 3 = 192; Zeile 4 = 0+24; Zeile 5 = 64+24; Zeile 6 = 128+24; Zeile 7 = 192+24
	unsigned char address = x + ((y % 4)*40);
	M50530_CS1_On;
	M50530_CS2_On;
	LCD_sendByte(address);
}
