/*
 * BMP180.h
 *
 * Created: 17.08.2015 10:13:01
 *  Author: buchin
 */ 
 
#include "stm32f0xx.h" // peripheral (CMSIS library)
#include "core_cm0.h" // F0 core
#include "delayf030.h" // delay subroutines, uses SysTick (defined in core_cm0.h)
#include "i2c_stm32f030f4.h"
#include "math.h"
//#include "debug.h"


#ifndef BMP180_H_
#define BMP180_H_

// DATA variables
 int32_t 	 BMP180_temperature = 0;
 int32_t   BMP180_pressure 		= 0;
 int32_t   BMP180_altitude 		= 0;

 int32_t BMP180_temperature_1 = 0;
 int32_t BMP180_pressure_1 		= 0;

// CALIBRATE variables
 int16_t BMP180_ac1;
 int16_t BMP180_ac2;
 int16_t BMP180_ac3;
 uint16_t BMP180_ac4;
 uint16_t BMP180_ac5;
 uint16_t BMP180_ac6;
 int16_t BMP180_b1;
 int16_t BMP180_b2;
 int16_t BMP180_mb;
 int16_t BMP180_mc;
 int16_t BMP180_md;
  
 
 int32_t tt0,t0,t1,t2,t3,t4,t5,t6;

// ========================================================================================================================
// ========================================================================================================================
// BMP180

// temp variables
uint32_t BMP180_D1;
uint32_t BMP180_D2;
uint32_t BMP180_D3;

// bmp180 definitions
#define BMP180_OSS 3
#define BMP180 0x77

// read 16 bit register
uint16_t BMP180_2byte_Read(uint8_t BMP180_address) {

	uint8_t BMP180_msb=0;
	uint8_t BMP180_lsb=0;
	uint16_t BMP180_data;
	I2C1_M_DISABLE();
	I2C1_MT_CFG(BMP180,1);
	I2C1_MT_1stbyte(BMP180_address);
	Delay_us(150); // wait for transfer complete
	I2C1_MR_CFG(BMP180,2);
	I2C1_M_START();
	BMP180_msb = I2C1_MR_RECEIVE();
	BMP180_lsb = I2C1_MR_RECEIVE();
	
	BMP180_data = (BMP180_msb << 8) + BMP180_lsb;
	return BMP180_data;

}

// get calibrating data
void BMP180_Calibration (void) {
//	union 
//	{
//		uint16_t WrdSigned;
//		int16_t WrdUsigned;
//	} BYTESx2;
//	
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xAA);
//	BMP180_ac1 = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xAC);
//	BMP180_ac2 = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xAE);
//	BMP180_ac3 = BYTESx2.WrdSigned ;
//	
//	BMP180_ac4 = BMP180_2byte_Read(0xB0);
//	BMP180_ac5 = BMP180_2byte_Read(0xB2);
//	BMP180_ac6 = BMP180_2byte_Read(0xB4);
//	
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xB6);
//	BMP180_b1  = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xB8);
//	BMP180_b2  = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xBA);
//	BMP180_mb  = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xBC);
//	BMP180_mc  = BYTESx2.WrdSigned ;
//	BYTESx2.WrdUsigned = BMP180_2byte_Read(0xBE);
//	BMP180_md  = BYTESx2.WrdSigned ;
	
	BMP180_ac1 = BMP180_2byte_Read(0xAA);
	BMP180_ac2 = BMP180_2byte_Read(0xAC);
	BMP180_ac3 = BMP180_2byte_Read(0xAE);
	BMP180_ac4 = BMP180_2byte_Read(0xB0);
	BMP180_ac5 = BMP180_2byte_Read(0xB2);
	BMP180_ac6 = BMP180_2byte_Read(0xB4);
	BMP180_b1  = BMP180_2byte_Read(0xB6);
	BMP180_b2  = BMP180_2byte_Read(0xB8);
	BMP180_mb  = BMP180_2byte_Read(0xBA);
	BMP180_mc  = BMP180_2byte_Read(0xBC);
	BMP180_md  = BMP180_2byte_Read(0xBE);

}

// read atm. pressure
void BMP180_get_temper(void){

	I2C1_MT_CFG(BMP180,2);
	I2C1_MT_1stbyte(0xF4);	
	I2C1_MT_nextbyte(0x2E);	
	
	Delay_ms(30);                   // measure (5 - 26 ms depend on (oss))

	BMP180_temperature_1 = BMP180_2byte_Read(0xF6);     // calc temperature

}

// read atm. pressure
void BMP180_get_pressure(void){

	I2C1_MT_CFG(BMP180,2);
	I2C1_MT_1stbyte(0xF4);	
	I2C1_MT_nextbyte(0x34+(BMP180_OSS<<6));	

	Delay_ms(100);                  // measure (5 - 26 ms (oss))

	I2C1_MT_CFG(BMP180,1);
	I2C1_MT_1stbyte(0xF6);	
	
	Delay_us(150);
	
	I2C1_MR_CFG(BMP180,3);
	I2C1_M_START();
	BMP180_D1=I2C1_MR_RECEIVE();
	BMP180_D2=I2C1_MR_RECEIVE();
	BMP180_D3=I2C1_MR_RECEIVE();

	BMP180_pressure_1 = ((BMP180_D1 << 16) + (BMP180_D2 << 8) + BMP180_D3) >> (8-BMP180_OSS);  // calc pressure

}

// calc temper + pressure in coord with calibration values
void BMP180_calculation (int32_t* BMP180_temperature, int32_t* BMP180_pressure) {

	//int8_t i;
	int32_t BMP180_ut=0;
	int32_t BMP180_up=0;
	int32_t BMP180_x1, BMP180_x2, BMP180_b5, BMP180_b6, BMP180_x3, BMP180_b3, BMP180_p;
	uint32_t BMP180_b4, BMP180_b7;

	BMP180_get_temper();
	BMP180_ut+=BMP180_temperature_1;

	BMP180_get_pressure();
	BMP180_up=BMP180_pressure_1;

	BMP180_x1 = ((int32_t)BMP180_ut - (int32_t)BMP180_ac6) * (int32_t)BMP180_ac5 >> 15;
	BMP180_x2 = ((int32_t)BMP180_mc << 11) / (BMP180_x1 + BMP180_md);
	BMP180_b5 = BMP180_x1 + BMP180_x2;
	*BMP180_temperature = (BMP180_b5 + 8) >> 4;
	
	BMP180_b6 = BMP180_b5 - 4000;
	BMP180_x1 = (BMP180_b2 * ((BMP180_b6 * BMP180_b6) >> 12)) >> 11;
	BMP180_x2 = (BMP180_ac2 * BMP180_b6) >> 11;
	BMP180_x3 = BMP180_x1 + BMP180_x2;
	BMP180_b3 = (((((int32_t) BMP180_ac1) * 4 + BMP180_x3)<<BMP180_OSS) + 2)>> 2;
	BMP180_x1 = (BMP180_ac3 * BMP180_b6) >> 13;
	BMP180_x2 = ((int32_t)BMP180_b1 * ((BMP180_b6 * BMP180_b6) >> 12))/65536;// >> 16;
	BMP180_x3 = ((BMP180_x1 + BMP180_x2) + 2) >> 2;
	BMP180_b4 = (BMP180_ac4 * (uint32_t) (BMP180_x3 + 32768)) >> 15;
	BMP180_b7 = ((uint32_t) (BMP180_up - BMP180_b3) * (50000 >> BMP180_OSS));
	//BMP180_p = BMP180_b7 < 0x80000000 ? (BMP180_b7 * 2) / BMP180_b4 : (BMP180_b7 / BMP180_b4) * 2;
	
	if (BMP180_b7 < 0x80000000)
	{
		BMP180_p = (BMP180_b7 *2/*<< 1*/) / BMP180_b4;
	}
	else
	{
		BMP180_p = (BMP180_b7 / BMP180_b4) *2;//<< 1;
	}

	BMP180_x1 = (BMP180_p >> 8) * (BMP180_p >> 8);
	BMP180_x1 = (BMP180_x1 * 3038) >> 16;
	BMP180_x2 = (-7357 * BMP180_p) >> 16;
	*BMP180_pressure = BMP180_p + ((BMP180_x1 + BMP180_x2 + 3791) >> 4);

}


// calc altitude
void bmp180CalcAltitude(int32_t BMP180_pressure){
	float BMP180_temp;

	BMP180_temp = (float) BMP180_pressure/101325;
	BMP180_temp = 1-pow(BMP180_temp, 0.19029);
	//BMP180_altitude = round(44330*BMP180_temp*10);
	BMP180_altitude = 44330*BMP180_temp*100;
	//get altitude in dm
}

// ========================================================================================================================
// ========================================================================================================================




#endif /* BMP180_H_ */
