/*
 * HTU21D.h
 *
 * Created: 17.08.2015 10:46:04
 *  Author: buchin
 */ 

#include "stm32f0xx.h" // peripheral (CMSIS library)
#include "core_cm0.h" // F0 core
#include "delayf030.h" // delay subroutines, uses SysTick (defined in core_cm0.h)
#include "i2c_stm32f030f4.h"
#include "GPIOmacro.h"

#ifndef HTU21D_H_
#define HTU21D_H_


// DATA variables
volatile int32_t HTU21D_temperature = 0;
volatile int32_t HTU21D_humidity	= 0;



// ========================================================================================================================
// ========================================================================================================================
// HTU21D

#define HTU21D 0x40

// HTU21D commands
#define HTU21D_TrigTemperMeasHoldClk	0xE3
#define HTU21D_TrigHumidityMeasHoldClk	0xE5
#define HTU21D_TrigTemperMeasNoHold		0xF3
#define HTU21D_TrigHumidityMeasNoHold	0xF5
#define HTU21D_WriteUserRegister		0xE6
#define HTU21D_ReadUserRegister			0xE7
#define HTU21D_SoftReset				0xFE

#define HTU21D_Resolution_Rh12_T14 0
#define HTU21D_Resolution_Rh08_T12 1
#define HTU21D_Resolution_Rh10_T13 2
#define HTU21D_Resolution_Rh11_T11 3
#define HTU21D_Resolution_NoChange 4

#define HTU21D_HeaterDisable	0
#define HTU21D_HeaterEnable		1
#define HTU21D_HeaterNoChange	2

#define HTU21D_OTPDisable	0
#define HTU21D_OTPEnable	1
#define HTU21D_OTPNoChange	2


// read 16 bit register
int32_t HTU21D_ReadTemperature(void)
{
	int32_t HTU21D_temperature_RAW;
	int32_t HTU21D_temperature;
	uint32_t HTU21D_temperature1;
	uint32_t HTU21D_temperature2;
	//int8_t HTU21D_temperatureCRC;
	
	I2C1_MT_CFG(HTU21D,1);
	I2C1_MT_1stbyte(HTU21D_TrigTemperMeasNoHold);
	Delay_ms(60);
	I2C1_MR_CFG(HTU21D,2);
	I2C1_M_START();
	HTU21D_temperature1=I2C1_MR_RECEIVE();
	HTU21D_temperature2=I2C1_MR_RECEIVE();
//	temperatureCRC=I2C1_MR_RECEIVE(); // need to change number of bytes in I2C1_MR_CFG(0x40,2)
	HTU21D_temperature_RAW = (((HTU21D_temperature1)<<8)+HTU21D_temperature2);     // calc temperature
	HTU21D_temperature = (((17572*HTU21D_temperature_RAW)>>16)-4685);
	return HTU21D_temperature;
}

int32_t HTU21D_ReadHumidity(void)
{
	int32_t HTU21D_RH;
	uint32_t HTU21D_humidity1;
	uint32_t HTU21D_humidity2;
	//int8_t HTU21D_humidity                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   CRC;

	I2C1_MT_CFG(HTU21D,1);
	I2C1_MT_1stbyte(HTU21D_TrigHumidityMeasNoHold);
	Delay_ms(20);
	I2C1_MR_CFG(HTU21D,2);
	I2C1_M_START();
	HTU21D_humidity1=I2C1_MR_RECEIVE();
	HTU21D_humidity2=I2C1_MR_RECEIVE();
//	HTU21D_humidityCRC=I2C1_MR_RECEIVE();// need to change number of bytes in I2C1_MR_CFG(0x40,2)

	HTU21D_humidity = (((HTU21D_humidity1)<<8)+HTU21D_humidity2);     // calc temperature
	HTU21D_RH=(-600+((12500*HTU21D_humidity)>>16));
	return HTU21D_RH;
}

void HTU21D_SoftResetSubr(void)
{
	I2C1_MT_CFG(HTU21D,1);
	I2C1_MT_1stbyte(HTU21D_SoftReset);
	Delay_ms(500);								// wait for reset routine (15 ms)
	return ;
}

uint16_t HTU21D_RWUserRegister(uint8_t HTU21D_RHresolution, uint8_t HTU21D_HeaterEnableReg, uint8_t HTU21D_OTPEnableReg)
{
	uint8_t HTU21D_UserReg ;
	uint8_t HTU21D_UserRegNew ;
	
	I2C1_MT_CFG_noSTOP(HTU21D,1);
	I2C1_MT_1stbyte(HTU21D_ReadUserRegister);
	
	Delay_us(50); // wait for transfer complete
	
	I2C1_MR_CFG(HTU21D,1);
	I2C1_M_START();
	HTU21D_UserReg = I2C1_MR_RECEIVE();
	
	HTU21D_UserRegNew = HTU21D_UserReg;
	if (HTU21D_OTPEnableReg==HTU21D_OTPEnable){HTU21D_UserRegNew &= ~(1<<1);}
	if (HTU21D_OTPEnableReg==HTU21D_OTPDisable){HTU21D_UserRegNew |= (1<<1);}

	if (HTU21D_HeaterEnableReg==HTU21D_HeaterDisable) {HTU21D_UserRegNew &= ~(1<<2);}
	if (HTU21D_HeaterEnableReg==HTU21D_HeaterEnable)  {HTU21D_UserRegNew |=  (1<<2);}

	switch (HTU21D_RHresolution)
	{
	case HTU21D_Resolution_Rh12_T14:{HTU21D_UserRegNew &=~(1<<0);	HTU21D_UserRegNew &=~(1<<7);	break;}
	case HTU21D_Resolution_Rh08_T12:{HTU21D_UserRegNew |=(1<<0);	HTU21D_UserRegNew &=~(1<<7);	break;}
	case HTU21D_Resolution_Rh10_T13:{HTU21D_UserRegNew &=~(1<<0);	HTU21D_UserRegNew |=(1<<7);		break;}
	case HTU21D_Resolution_Rh11_T11:{HTU21D_UserRegNew |=(1<<0);	HTU21D_UserRegNew |=(1<<7);		break;}
	default:						{break;}
	}

	I2C1_MT_CFG(HTU21D,2);
	I2C1_MT_1stbyte(HTU21D_WriteUserRegister);
	I2C1_MT_nextbyte(HTU21D_UserRegNew);
	return ((HTU21D_UserReg<<8) + HTU21D_UserRegNew);
}


// ========================================================================================================================
// ========================================================================================================================



#endif /* HTU21D_H_ */
