// Using CMSIS 
// STM32F100C6T6, S6D0129 display, ADC buttons

#include "stm32f10x.h"
#include "main.h"
#include "math.h"
#include "DelaySysTick.h"	// dummy delays using systick timer
#include "lcd_s6d0129.h"  // china display 320x240 based on s6d0129 controller driver
#include "GPIO_macro.h"

#define F_PCLK1_MHz 24  // max 24
#define F_PCLK2_MHz 24  // max 24

#define BaudUSART1 1000000UL // baud rate of usart1 hundreds
#define BaudUSART1BRR (F_PCLK2_MHz*1000000/BaudUSART1)

/* Error codes used to make the orange led blinking */
#define ERROR_OVERFLOW 0x01
#define ERROR_UNEXPECTED_IT 0x02
#define CONFIG_MODE_F 0xAA
#define CONFIG_MODE_T 0x55
#define MODE_SELECTED 0x00

#define BUTTON1 0x0010
#define BUTTON2 0x0040
#define BUTTON3 0x0100
#define BUTTON4 0x0400
#define BUTTON5 0x1000


static __IO uint32_t TimingDelay;
const int hex[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

uint32_t iii=0, RawDuty, RawPeriod;
uint8_t i001,i010,i100,i1k,i10k,i100k,i1M,i10M,i100M,i1G;
uint8_t upd_display=TRUE, countFnotT=TRUE, error, mode=MODE_SELECTED; 
uint32_t i32, jjj=0, kkk=0;
volatile uint16_t button_voltage=0, button_code;

u32 TrueFreq;
volatile u32 RawFreq ;
volatile FlagStatus DataReady = RESET;
volatile FlagStatus DataReadyP = RESET;

unsigned char bcd (unsigned char data)
{
	unsigned char bc;

	bc=((((data&(1<<6))|(data&(1<<5))|(data&(1<<4)))*0x0A)>>4)+((data&(1<<3))|(data&(1<<2))|(data&(1<<1))|(data&0x01));

	return bc;
}
//

void InitRCC(void)
{
	if((RCC->CR & RCC_CR_HSIRDY) == 0 ){RCC->CR |=RCC_CR_HSION;} 	// if HSI OFF -> HSI ON 
	while (!(RCC->CR & RCC_CR_HSIRDY)){;};												// wait HSI ON	
	RCC->CFGR |= RCC_CFGR_SW_HSI; 													// set working from HSI
	while ((RCC->CFGR&RCC_CFGR_SWS)!=RCC_CFGR_SWS_HSI){;}; 	// wait HSI tact start	

	RCC->CR |= RCC_CR_HSEON;																// start external high speed oscillator
	while (!(RCC->CR & RCC_CR_HSERDY)){;};									// wait HSE ON

	RCC->CFGR |= RCC_CFGR_SW_HSE; 													// set working from HSE
	while ((RCC->CFGR&RCC_CFGR_SWS)!=RCC_CFGR_SWS_HSE){;}; 	// wait HSE tact start	

	FLASH->ACR |= FLASH_ACR_PRFTBE ;				// allow buffer prefetch from FLASH (for high spees working)
	FLASH->ACR &= ~FLASH_ACR_LATENCY ;
	FLASH->ACR |= FLASH_ACR_LATENCY_2 ;	
	
	RCC->CFGR2 &= ~RCC_CFGR2_PREDIV1; 		// reset PREDIV1 prescaler PLLinCLK = HSE/1
	RCC->CFGR2 |= RCC_CFGR2_PREDIV1_DIV2; // PREDIV1 prescaler PLLinCLK = HSE/2
	RCC->CFGR |= RCC_CFGR_HPRE_DIV1; 			// AHB  prescaler HCLK  = SYSCLK
	RCC->CFGR |= RCC_CFGR_PPRE2_DIV1; 		// APB2 prescaler PCLK2 = HCLK/2
	RCC->CFGR |= RCC_CFGR_PPRE1_DIV1; 		// APB1 prescaler PCLK1 = HCLK/2
	RCC->CFGR &= ~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLMULL );	
	RCC->CFGR |= RCC_CFGR_PLLSRC_PREDIV1 ; // PLL from HSE followed by PREDIV1 divider
	//RCC->CFGR |= RCC_CFGR_PLLSRC_HSI_Div2 ; // PLL from HSI
	RCC->CFGR |= RCC_CFGR_PLLMULL6 ; 	 // PLL mull=6 SYSCLK = HSE/2(4MHz)*6=24
		
	RCC->CR |= RCC_CR_PLLON; // switch PLL on
	while ((RCC->CR & RCC_CR_PLLRDY)==0) {;};//wait for PLL ON  	
	
	RCC->CFGR &=~RCC_CFGR_SW; 							//	clear SW bits
	RCC->CFGR |= RCC_CFGR_SW_PLL; 					//	set working from PLL
	while ((RCC->CFGR&RCC_CFGR_SWS)!=RCC_CFGR_SWS_PLL){;}; //wait PLL tact start

	RCC->CR |= RCC_CR_CSSON ; // syncro protection ON
}
//

void InitPorts(void)
{
  RCC->APB2ENR	|= RCC_APB2ENR_IOPAEN;// enable clock to port A
  RCC->APB2ENR	|= RCC_APB2ENR_IOPBEN;// enable clock to port B
  RCC->APB2ENR	|= RCC_APB2ENR_IOPCEN;// enable clock to port C
	RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;	// enable clock to alternate port functions
	
	GPIOA->CRL  = 0x00000000;
	GPIOA->CRH  &= (GPIO_CRH_CNF13 | GPIO_CRH_CNF14 | GPIO_CRH_MODE13 | GPIO_CRH_MODE14); // leave SWD programming enabled
	GPIOB->CRL  = 0x00000000;
	GPIOB->CRH  = 0x00000000;
	GPIOC->CRL  = 0x00000000;	
	GPIOC->CRH  = 0x00000000;	

	GPIOA->CRH	&= ~GPIO_CRH_CNF12;			// PA12 				Push-Pull (00),  S6D0129 nCS
	GPIOA->CRH |= GPIO_CRH_MODE12;   		// PA12 					OUT 50MHz (11)
	GPIOA->CRH	&= ~GPIO_CRH_CNF11;			// PA11 				Push-Pull (00),  S6D0129 RS
	GPIOA->CRH |= GPIO_CRH_MODE11;   		// PA11 					OUT 50MHz (11)
	GPIOA->CRH	&= ~GPIO_CRH_CNF10;			// PA10 				Push-Pull (00),  S6D0129 nWR
	GPIOA->CRH |= GPIO_CRH_MODE10;   		// PA10 					OUT 50MHz (11)
	GPIOA->CRH	&= ~GPIO_CRH_CNF9;			// PA9 					Push-Pull (00),  S6D0129 nRD
	GPIOA->CRH |= GPIO_CRH_MODE9;   		// PA9 						OUT 50MHz (11)
	GPIOA->CRH	&= ~GPIO_CRH_CNF8;			// PA8	 				Push-Pull (00),  S6D0129 nLED
	GPIOA->CRH |= GPIO_CRH_MODE8;   		// PA8	 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF15;			// PB15 				Push-Pull (00),  S6D0129 IM3
	GPIOB->CRH |= GPIO_CRH_MODE15;   		// PB15 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF14;			// PB14 				Push-Pull (00),  S6D0129 DB9
	GPIOB->CRH |= GPIO_CRH_MODE14;   		// PB14 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF13;			// PB13 				Push-Pull (00),  S6D0129 DB10
	GPIOB->CRH |= GPIO_CRH_MODE13;   		// PB13 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF12;			// PB12 				Push-Pull (00),  S6D0129 DB11
	GPIOB->CRH |= GPIO_CRH_MODE12;   		// PB12 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF11;			// PB11 				Push-Pull (00),  S6D0129 DB12
	GPIOB->CRH |= GPIO_CRH_MODE11;   		// PB11 					OUT 50MHz (11)
	GPIOB->CRH	&= ~GPIO_CRH_CNF10;			// PB10 				Push-Pull (00),  S6D0129 DB13
	GPIOB->CRH |= GPIO_CRH_MODE10;   		// PB10 					OUT 50MHz (11)
	GPIOB->CRL	&= ~GPIO_CRL_CNF2;			// PB2 					Push-Pull (00),  S6D0129 DB14
	GPIOB->CRL |= GPIO_CRL_MODE2;   		// PB2 						OUT 50MHz (11)
	GPIOB->CRL	&= ~GPIO_CRL_CNF1;			// PB1 					Push-Pull (00),  S6D0129 DB15
	GPIOB->CRL |= GPIO_CRL_MODE1;   		// PB1 						OUT 50MHz (11)
	GPIOB->CRL	&= ~GPIO_CRL_CNF0;			// PB0 					Push-Pull (00),  S6D0129 DB16
	GPIOB->CRL |= GPIO_CRL_MODE0;   		// PB0 						OUT 50MHz (11)
	GPIOA->CRL	&= ~GPIO_CRL_CNF7;			// PA7 					Push-Pull (00),  S6D0129 DB17
	GPIOA->CRL |= GPIO_CRL_MODE7;   		// PA7 						OUT 50MHz (11)
	GPIOA->CRL	&= ~GPIO_CRL_CNF6;			// PA6 					Push-Pull (00),  S6D0129 nRST
	GPIOA->CRL |= GPIO_CRL_MODE6;   		// PA6 						OUT 50MHz (11)

	GPIOA->CRL &= ~GPIO_CRL_CNF4;				// PA4 					Push-Pull (00),  LED
	GPIOA->CRL |= GPIO_CRL_MODE4;   		// PA4 						OUT 50MHz (11)
	GPIOA->CRL &= ~GPIO_CRL_CNF5;				// PA5 					Push-Pull (00),  LED
	GPIOA->CRL |= GPIO_CRL_MODE5;   		// PA5 						OUT 50MHz (11)
	GPIOA->CRL |= GPIO_BSRR_BR4 | GPIO_BSRR_BR5 ; 			// LED off

	GPIOA->CRL |= GPIO_CRL_CNF0_0;			// PA0					Alternate (01) input, ETR input clock for TIM2

	return;
}
//

void InitUSART(void)
{
  RCC->APB2ENR	|= RCC_APB2ENR_IOPBEN;					// enable clock to port A
 	RCC->APB2ENR |= RCC_APB2ENR_AFIOEN ; 					// enable alternate func clock
	RCC->APB2ENR |= RCC_APB2ENR_USART1EN ;				// enable USART1 clock
	AFIO->MAPR |= AFIO_MAPR_USART1_REMAP ;				// remap USART1 to PB6,PB7 pins
	
	GPIOB->CRL |= GPIO_CRL_MODE6_1 ;							// PB6 output push-pull 2 MHz
	GPIOB->CRL &= ~GPIO_CRL_MODE6_0 ;							// PB6 output push-pull 2 MHz
	GPIOB->CRL &= ~GPIO_CRL_CNF6_0 ;							// PB6 alternate function (USART TX)
	GPIOB->CRL |= GPIO_CRL_CNF6_1 ;								// PB6 alternate function (USART TX)
	
	GPIOB->CRL &= ~GPIO_CRL_MODE7_1 ;						// PB7 input alternate function
	GPIOB->CRL &= ~GPIO_CRL_MODE7_0 ;						// PB7 input alternate function
	GPIOB->CRL &= ~GPIO_CRL_CNF7_1 ;						// PB7 alternate function (USART RX)
	GPIOB->CRL |= GPIO_CRL_CNF7_0 ;							// PB7 alternate function (USART RX)

	USART1->CR1 |=(USART_CR1_RE | USART_CR1_TE); 	// USART1 enable Tx & Rx 
	USART1->BRR = BaudUSART1BRR ;									// Set baudrate
	USART1->CR1 |= USART_CR1_UE | USART_CR1_PCE | USART_CR1_M ; // Parity on , USART1 ON
	return;
}
//

unsigned int USART1RXTX(unsigned int txbyte) // 
{
	unsigned int rxbyte =0xFF00;
	if (USART1->SR & USART_SR_RXNE){rxbyte=USART1->DR;}
	
	if((txbyte&0xFF00)==0) //transmit byte if needeed 
	{
	 while (!(USART1->SR & USART_SR_TXE)){;};
	 USART1->DR=(uint8_t)(0xFF & txbyte); 
	}
	return rxbyte;
}
//

void USART1_TX(unsigned char txbyte) // 
{
	 while (!(USART1->SR & USART_SR_TXE)){;};
	 USART1->DR=(uint8_t)(txbyte); 
	
	return ;
}
//

void USART1_TXs(const char *s)
{
	while (*s) 
	{
		USART1_TX(*s);
		++s;
	}
}
//

void EXT_Timer2_ResetRegisters(void)
{

	RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;
	TIM2->CR1 &= ~TIM_CR1_CEN;
	TIM2->CR1 =0x0000;
	TIM2->CR2 =0x0000;
	TIM2->SMCR =0x0000;
	TIM2->DIER =0x0000;
	TIM2->SR =0x0000;
	TIM2->EGR =0x0000;
	TIM2->CCMR1 =0x0000;
	TIM2->CCMR2 =0x0000;
	TIM2->CCER =0x0000;
	TIM2->CNT =0x0000;
	TIM2->PSC =0x0000;
	TIM2->ARR =0x0000;
	TIM2->CCR1 =0x0000;
	TIM2->CCR2 =0x0000;
	TIM2->CCR3 =0x0000;
	TIM2->CCR4 =0x0000;
	TIM2->DCR =0x0000;
	TIM2->DMAR =0x0000;
	return;
	
}
//

void EXT_PeriodCounterConfig(void)
{
	EXT_Timer2_ResetRegisters();
	// TIM2 count period and duty on TI1 input
	RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;
	TIM2->CR1 &= ~TIM_CR1_CEN;
	TIM2->PSC = 240-1; // TIM2 clock 100 kHz
	TIM2->ARR = 0xffff;
//	TIM2->CR1 |= TIM_CR1_OPM; //counter stops counting at the next update event
	TIM2->CCMR1 |= TIM_CCMR1_CC1S_0 | TIM_CCMR1_CC2S_1;
// TIM2->SMCR |= TIM_SMCR_TS_2 | TIM_SMCR_TS_0 | TIM_SMCR_SMS_2; //
 TIM2->SMCR |= TIM_SMCR_TS_2 | TIM_SMCR_TS_0 ; //
 TIM2->SMCR |=  TIM_SMCR_SMS_2; //
	TIM2->CCER  |= TIM_CCER_CC2P | TIM_CCER_CC2E | TIM_CCER_CC1E ;
	TIM2->DIER |= TIM_DIER_CC1IE;//enable interrupt

	return;
}
//

void EXT_PeriodCounterRun(void)
{
  NVIC_DisableIRQ(TIM3_IRQn);
  NVIC_EnableIRQ(TIM2_IRQn);
  TIM3->CR1       &= ~TIM_CR1_CEN;
  TIM1->CR1       &= ~TIM_CR1_CEN;
  TIM2->CR1       |= TIM_CR1_CEN;
	RCC->APB1ENR &= ~RCC_APB1ENR_TIM3EN;
	RCC->APB2ENR &= ~RCC_APB2ENR_TIM1EN;
	RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;
	
	
}
//

void TIM2_IRQHandler(void)
{
	if((TIM2->SR & (TIM_SR_CC1IF | TIM_SR_CC1OF))==(TIM_SR_CC1IF))
	{
		if(TIM2->CCR1 !=0)
		{
			RawPeriod = (uint32_t)TIM2->CCR1;
			RawDuty = (uint32_t)TIM2->CCR2;
			DataReadyP = SET;
			error = 0;
			jjj++;
		}
	}
	else if ((TIM2->SR & TIM_SR_CC1OF) != 0)  /* Check the overflow */
	{
		error = ERROR_OVERFLOW;
		DataReadyP = SET;
		TIM2->SR &= ~(TIM_SR_CC1OF | TIM_SR_CC1IF); /* Clear the flags */
		return;
	}
	else
  {
    error = ERROR_UNEXPECTED_IT; /* Report an error */
  }
	return;
}
//

void EXT_FrequencyCounterConfig(void)                                 
{
	// TIM3 master counter, period 0.1, 1, 10, 100 sec
	EXT_Timer2_ResetRegisters();

	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;
	TIM3->CR1 &= ~TIM_CR1_CEN;
  NVIC_DisableIRQ(TIM3_IRQn);
	TIM3->PSC = 2400-1; // TIM3 clock 10 kHz
	TIM3->ARR = 10000-1; // TIM3 count 10000 so period 1 sec
	TIM3->CR1 |= TIM_CR1_DIR; // count down to 0
	TIM3->CR1 |= TIM_CR1_OPM; //counter stops counting at the next update event
	TIM3->CR2 |= TIM_CR2_MMS_0; //COUNTER_ENABLE signal to TIM1, used as trigger output (TRGO)
	TIM3->DIER |= TIM_DIER_UIE; //enable interrupt
	
	// TIM2 slave counter, count impulses from TIM2_ETR
	RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;
	TIM2->CR1 &= ~TIM_CR1_CEN;
  NVIC_DisableIRQ(TIM2_IRQn);
  TIM2->PSC = 0;
  TIM2->ARR = 0xFFFF; //counter max value
	TIM2->CR1 = 0 ;
  TIM2->CR2 = TIM_CR2_MMS_1; //update event is selected as trigger output to TIM1
  TIM2->SMCR = TIM_SMCR_ECE | TIM_SMCR_TS_1 | TIM_SMCR_SMS_0 | TIM_SMCR_SMS_2; //ext. clock mode2 enabled, counter is clocked by ETRF signal
	// TIM1 additional counter for TIM2 to make 32 bit counter
	RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;
	TIM1->CR1 &= ~TIM_CR1_CEN;
	TIM1->PSC = 0;
  TIM1->ARR = 0xFFFF; //counter max value
  TIM1->CR1  &= ~TIM_CR1_DIR; //used as upcounter
  TIM1->SMCR &= ~TIM_SMCR_ETPS; //no external trigger prescaller
  TIM1->SMCR &= ~TIM_SMCR_ETF; //no external trigger filter
	TIM1->SMCR |=TIM_SMCR_TS_0;
  TIM1->SMCR |= TIM_SMCR_SMS_0 | TIM_SMCR_SMS_1 | TIM_SMCR_SMS_2; //TIM1 (TRGI) clock the counter
	
	return;
}
//

void EXT_FrequencyCounterRun(void)
{
  NVIC_DisableIRQ(TIM2_IRQn);
  NVIC_EnableIRQ(TIM3_IRQn);
  TIM1->CR1       |= TIM_CR1_CEN;
  TIM2->CR1       |= TIM_CR1_CEN;
  TIM3->CR1       |= TIM_CR1_CEN;
}
//

void TIM3_IRQHandler(void)
{
  TIM3->SR &= ~TIM_SR_UIF; //reset interrupt flag

  RawFreq = ((u32)(TIM1->CNT) << 16) | ((u32)(TIM2->CNT));
  DataReady = SET;
  TIM2->CNT = 0;
  TIM1->CNT = 0;

  TIM3->CR1 |= TIM_CR1_CEN; //another circle
	iii++;
}
//

void TIM7_ADC_ButtonCheckConfig(void)
{
	RCC->APB2ENR |= RCC_APB2ENR_ADC1EN;
	ADC1->SMPR2 = ADC_SMPR2_SMP1_0;
	ADC1->CR1 |= /*ADC_CR1_EOCIE |*/ ADC_CR1_DISCEN;
//  NVIC_EnableIRQ(ADC1_IRQn);
	ADC1->CR2 |= ADC_CR2_ADON | ADC_CR2_EXTTRIG | ADC_CR2_EXTSEL;
	ADC1->SQR1 = 0x0000;
	ADC1->SQR2 = 0x0000;
	ADC1->SQR3 |= ADC_SQR3_SQ1_0;
//	ADC1->CR2 |= ADC_CR2_CAL;
	
	RCC->APB1ENR |= RCC_APB1ENR_TIM7EN;
	TIM7->CR1 &= ~TIM_CR1_CEN;
	TIM7->PSC = 2400-1; // TIM7 clock 10 kHz
	TIM7->ARR = 1000-1; // TIM7 count 1000 so period 0.1 sec
	TIM7->CR1 |= TIM_CR1_DIR; // count down to 0
	TIM7->CR1 &= ~TIM_CR1_OPM; //counter not stops counting at the next update event
	TIM7->DIER |= TIM_DIER_UIE; //enable interrupt
	NVIC_EnableIRQ(TIM7_IRQn);
	TIM7->CR1 |= TIM_CR1_CEN; // start counter
	return;
}
//

void TIM7_IRQHandler(void)
{
	button_voltage=ADC1->DR;
	if(button_voltage>3600){button_code=FALSE;}
	else if(button_voltage>3000){button_code=BUTTON1;}
	else if(button_voltage>2300){button_code=BUTTON2;}
	else if(button_voltage>1600){button_code=BUTTON3;}
	else if(button_voltage>1000){button_code=BUTTON4;}
	else if(button_voltage>300){button_code=BUTTON5;}
	else {button_code=FALSE;}
	
	TIM7->SR &= ~TIM_SR_UIF; //reset interrupt flag
	ADC1->CR2 |= ADC_CR2_SWSTART;
	return;
}
//

void ADC_IRQHandler()
{
	ADC1->SR =0;
	return;
}
//

int main(void)
{
	InitPorts(); // CLEARING CRH,CRL of used ports must be called first before RCC configuration
	InitRCC();
	InitUSART();
	InitSysTick(); // using for Delay_ms function
	TIM7_ADC_ButtonCheckConfig();
	
	EXT_PeriodCounterConfig();
  EXT_PeriodCounterRun();
	countFnotT=FALSE;
	
	Delay_us(14030);	
	
	EXT_FrequencyCounterConfig();
  EXT_FrequencyCounterRun();
	countFnotT=TRUE;

	__enable_irq();
	
	LED_GOn;

	while(100)
	{
		if(button_code==FALSE){;}
		else if(button_code==BUTTON1)
		{
			__disable_irq();
			EXT_FrequencyCounterConfig();
			EXT_FrequencyCounterRun();
			countFnotT=TRUE;
			__enable_irq();;
		}
		else if(button_code==BUTTON2)
		{
			NVIC_ClearPendingIRQ(TIM2_IRQn);
		}
		else if (button_code==BUTTON3)
		{
			if(upd_display==TRUE){upd_display=FALSE;}
			else{upd_display=TRUE;}
		}
		else if(button_code==BUTTON4)
		{
//			NVIC_ClearPendingIRQ(TIM3_IRQn);
		}
		else if(button_code==BUTTON5)
		{
//			__disable_irq();
			EXT_PeriodCounterConfig();
			EXT_PeriodCounterRun();
			countFnotT=FALSE;
//			__enable_irq();;
		}
		

  }
	//while(100)
	
}
//main()

